<?php
class ControllerPaymentGpndata extends Controller {

	public function __construct()
	{
		global $registry;
		parent::__construct($registry);
        //GPN DATA API library - this is a workaround so extension is supported in v.2+ of opencart
		if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
			$file = DIR_SYSTEM . 'library\\gpndata.php';
		} else {
			$file = DIR_SYSTEM . 'library/gpndata.php';
		}
		//$file = DIR_SYSTEM . 'library\\gpndata.php';
		if (file_exists($file)){
			include_once($file);
		} else {
			trigger_error('Error: Could not load library ' . $file . '!');
			exit();
		}

		if($this->_db == null){
			$this->_db = new Database(DB_PREFIX, DB_HOSTNAME, DB_DATABASE, DB_USERNAME, DB_PASSWORD);
		}

		if($this->_API == null){
			$APIUser		= $this->config->get('gpndata_api_user');
			$APIPassword	= $this->config->get('gpndata_api_password');
			$APIKey			= $this->config->get('gpndata_api_key');

			$this->_API = new GPNDataAPI($APIUser, $APIPassword, $APIKey);
			$this->_API->paymentPageUrl = trim($this->config->get('gpndata_transaction_url'));
			$this->_API->transactionUrl = trim($this->config->get('gpndata_transaction_url_full_api'));
            $this->_API->merchantName	= $this->config->get('gpndata_merchant_name');
		}
	}

	public function index(){

		$this->load->language('payment/gpndata');

		$data['button_confirm_transaction']				= $this->language->get('button_confirm_transaction');
		$data['gpndata_payment_description']			= $this->config->get('gpndata_description');
		$data['error_payment_request_not_processed']	= $this->language->get('error_payment_request_not_processed');
		$data['error_payment_page_one']					= $this->language->get('error_payment_page_one');
		$data['error_payment_page_two']					= $this->language->get('error_payment_page_two');
		$data['error_payment_page_three']				= $this->language->get('error_payment_page_three');
		$data['gpndata_payment_description']			= $this->config->get('gpndata_description');

		return $this->load->view('default/template/payment/gpndata.tpl', $data);
	}

	public function send(){

		$this->load->language('payment/gpndata');
		$data['button_confirm_transaction'] = $this->language->get('button_confirm_transaction');
		$this->load->model('checkout/order');

        $order_id   = $this->session->data['order_id'];
		$order_info = $this->model_checkout_order->getOrder($order_id);

		if ($this->cart->hasShipping()) {
			$data['payment_firstname'] = $order_info['shipping_firstname'];
			$data['payment_lastname'] = $order_info['shipping_lastname'];
			$data['email'] = $order_info['email'];
			$data['address1'] = $order_info['shipping_address_1'];
			$data['address2'] = $order_info['shipping_address_2'];
			$data['city'] = $order_info['shipping_city'];
			$data['state'] = $order_info['payment_zone_code'];
			$data['country'] = $order_info['shipping_iso_code_3'];
			$data['postcode'] = $order_info['shipping_postcode'];

		}
		else{

			$data['payment_firstname'] = $order_info['payment_firstname'];
			$data['payment_lastname'] = $order_info['payment_lastname'];
			$data['email'] = $order_info['email'];
			$data['address1'] = $order_info['payment_address_1'];
			$data['address2'] = $order_info['payment_address_2'];
			$data['city'] = $order_info['payment_city'];
			$data['state'] = $order_info['payment_zone_code'];
			$data['country'] = $order_info['payment_iso_code_3'];
			$data['postcode'] = $order_info['payment_postcode'];
		}
		try
		{
			$products = $this->cart->getProducts();
			$merchanttransid = strval($order_id).'-'.time().rand(100, 999); // <-- required (see API doc)
			$successurl =  $this->url->link('checkout/success', '', 'SSL');

			$failureurl =  $this->url->link('payment/gpndata/fail', '', 'SSL');
			$contacturl = $this->config->get('gpndata_contact_url');
			$currency = $order_info['currency_code'];

			$transactionDetails = $this->_API->constructTransactionDetails($merchanttransid, $order_info['total'], $order_info['currency_code'], 'Payment OpenCart '.$merchanttransid);

			$customerDetails = $this->_API->constructCustomerDetails(
												$data['payment_firstname'],
												$data['payment_lastname'],
												$data['email'],
												$this->_API->getIpAddress(),
												$data['country'],
												$data['state'],
												$data['postcode'],
												$data['city'],
												$data['address1'],
												$data['address2'],
												$order_info['telephone'], //  phone,
												$data['email']);

			$productDetails = $this->_API->constructProductDetailsPaymentPage($products,
																 array(
                                                                    'name'          => 'name',
																	'description'   => 'name',
																	'price'         => 'price',
																	'amount'        => 'quantity'
																 ));

			$result = $this->_API->startCreditCardChargePaymentPage($transactionDetails, $customerDetails, $productDetails, $successurl, $failureurl, $contacturl);
		}
		catch (Exception $e)
		{
			echo 'Caught exception: ',  $e->getMessage(), "\n";
			die();
		}

		$json = array();
		if($result->result == 'SUCCESS'){
		    $json['success'] = $result->customerurl;
		}
		else {
		    $json['error']	= empty($result->errormessage) ? $result->description : $result->errormessage;
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
		//$this->response->output();

	}

	/** @var GPNDataAPI _API - this is the full API SDK class*/
	private $_API;
	/** @var Database _db - this is database manager class*/
	private $_db;

	public function notify() {

		if(isset($this->request->post['strdata'])){

			Header('Content-type: text/xml');
			$strData = htmlspecialchars_decode($this->request->post['strdata']);
			$xmldata = simplexml_load_string($strData);
			$this->handleNotification($xmldata);
		}
	}

	public function fail(){

		$this->load->language('payment/gpndata');
		$this->session->data['error'] = $this->language->get('payment_transaction_declined');
		$this->response->redirect($this->url->link('checkout/checkout', '', 'SSL'));
	}

	private function handleNotification($xmldata){

		$message = 'OK';
		if (empty($xmldata)){
			unset($xmldata);
			$resstr = '
			    <?xml version="1.0" encoding="utf-8" ?>
			    <transaction>
			    <result>ERROR</result >
			    <gatetransid></gatetransid>
			    <merchanttransid></merchanttransid>
			    <errorcode></errorcode>
			    <errormessage>No XML Data</errormessage>
			    </transaction>';
			echo $resstr;
			die();
		}

		$this->load->model('payment/gpndata');
		$this->load->model('checkout/order');
		$this->load->language('payment/gpndata');

		if ($this->_API->validateNotificationChecksum($xmldata)){

			switch ($xmldata->apiCmd) {
				case NotificationResponses::TransactionStateNotification://This is Transaction Notification

					$message = $this->processTransactionNotification($xmldata);
					break;
				case NotificationResponses::RefundNotification:

					$message = $this->processRefundNotification($xmldata);//This is Refund Notification
					break;
				case NotificationResponses::DisputeNotification: //This is Chargeback Notification

					$message = $this->processChargebackNotification($xmldata);
					break;
				default:
					unset($xmldata);
			}
		}
		else {
			unset($xmldata);
			$resstr = '
			    <?xml version="1.0" encoding="utf-8" ?>
			    <transaction>
			    <result>ERROR</result >
			    <gatetransid></gatetransid>
			    <merchanttransid></merchanttransid>
			    <errorcode></errorcode>
			    <errormessage>Invalid Checksum</errormessage>
			    </transaction>';
			echo htmlspecialchars($resstr);
			die();
		}

		$resstr = '
			<?xml version="1.0" encoding="utf-8" ?>
			<transaction>
				<result>OK</result>
				<plugin-message>'.$message.'</plugin-message>
				<gatetransid>'.$xmldata->gatetransid.'</gatetransid>
			</transaction>';
		echo $resstr;
		unset($xmldata);
	}

	public function processTransactionNotification($xmldata){

		$merchanttransid            = $xmldata->merchanttransid;
		$transaction_gatetransid    = $xmldata->gatetransid;
		$amount                     = $xmldata->amount;
		$curcode                    = $xmldata->curcode;
		$transaction_status         = $xmldata->status;
		$state                      = $xmldata->state;
		$description                = $xmldata->description;

		$message					= 'OK';
		$val						= explode('-', $merchanttransid);
		$order_id					= $val[0];
		$val						= null;
		$order_exits				= $this->_db->orderExists($order_id);
		if(!$order_exits){
			$orderData = array(
								'order_id'			=> (int)$order_id,
								'merchant_trans_id'	=> "$merchanttransid",
								'gate_trans_id'		=> "$transaction_gatetransid",
								'curcode'			=> "$curcode",
								'total_amount'		=> "$amount"
								);

			$this->_db->addOrder($orderData);
		}
		$text_transaction_status_by_notification = $this->language->get('text_transaction_status_by_notification');
		$transactionData			= array(
											'order_id'			=> (int)$order_id,
											'status'			=> "$state",
											'state'				=> "$state",
											'description'		=> "$description",
											'amount'			=> "$amount",
											'additional_text'	=> "$text_transaction_status_by_notification"
											);
		switch ($state){
			case TransactionStatus::PENDING:
				$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_pending_id')), $description);
				break;
			case TransactionStatus::DECLINED:
				$this->_db->updateOrder(array('order_id' => $order_id, 'is_declined' => 1));
				$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_declined_id')), $description);
				break;
			case TransactionStatus::AUTHORIZED:
				$this->_db->updateOrder(array('order_id' => $order_id, 'is_authorized' => 1));
				$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_pending_id')), $description);
				break;
			case TransactionStatus::CAPTURED:
				$this->_db->updateOrder(array('order_id' => $order_id, 'is_captured' => 1));
				$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_captured_id')), $description);
				break;
			case TransactionStatus::CANCELED:
				$this->_db->updateOrder(array('order_id'=> $order_id, 'is_voided' => 1));
				$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_void_id')), $description);
				break;
			default:
				break;
		}

		$this->_db->addTransaction($transactionData);
		if(strlen($merchanttransid) == 0 || strlen($transaction_gatetransid) == 0)
			$message = 'Notification data is missing! Fields merchant transaction id or gateway transaction are empty. This needs to be considered as an ERROR! OpenCart Order status has not been updated accordingly!';
		else
			$message = "Order transaction row successfully created with status $state !";

		return $message;
	}

	private function processRefundNotification($xmldata){

		$merchanttransid			= $xmldata->merchanttransid;
		$transaction_gatetransid	= $xmldata->gatetransid;
		$amount						= $xmldata->amount;
		$state						= $xmldata->status;

		$val						= explode('-', $merchanttransid);
		$order_id					= $val[0];
		$val						= null;
		$transaction				= $this->_db->getOrder((int)$order_id);
		$transaction_status			= null;
		$description				= null;
		$curcode					= $transaction['curcode'];

		if($state == TransactionStatus::SUCCESS){
			$transaction_status = TransactionStatus::REFUNDED;
			$description = $this->language->get('text_refunded_amount'). $amount. ' '.$transaction['curcode'];
		}
		else{
			$transaction_status = 'REFUND DECLINED';
			$description = 'Refund request DECLINED '.$this->_API->formatAmount($amount).' '.$curcode;
		}

		$refunded_amount = $transaction_status == TransactionStatus::REFUNDED ? $amount : 0.00;
		$text_transaction_status_by_notification = $this->language->get('text_transaction_status_by_notification');
		$transactionData = array('order_id'			=> (int)$order_id,
								 'status'			=> "$state",
								 'state'			=> "$state",
								 'description'		=> "$description",
								 'amount'			=> "$refunded_amount",
								 'additional_text'	=> "$text_transaction_status_by_notification"
								 );

		$this->_db->updateOrder(array('order_id'=> $order_id, 'is_refund_pending' => 0, 'is_refunded' => 1));
		$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_refund_id')), $description);
		$this->_db->addTransaction($transactionData);
		if(strlen($merchanttransid) == 0 || strlen($transaction_gatetransid) == 0)
			$message = 'Notification data is missing! Fields merchant transaction id or gateway transaction are empty. This needs to be considered as an ERROR! OpenCart Order status has not been updated accordingly!';
		else
			$message = 'Refund notification updated transaction log table successfully!';

		return $message;
	}

	private function processChargebackNotification($xmldata){

		$transaction_gatetransid	= $xmldata->gatetransid;
		$chgbk_transref				= $xmldata->transref;
		$merchanttransid			= $xmldata->merchanttransid;
		$state						= $xmldata->status;
		$amount						= $xmldata->amount;
		$chgbk_currency				= $xmldata->currency;
		$chgbk_chbDate				= $xmldata->chbDate; //not used
		$chgbk_reasonCode			= $xmldata->reasonCode;
		$chgbk_reasonText			= $xmldata->reasonText;
		$chgbk_chbRef				= $xmldata->chbRef;
		$curcode					= $xmldata->currency;
		$val						= explode('-', $merchanttransid);
		$order_id					= $val[0];
		$description				= $chgbk_reasonText.'. Reason code: '.$chgbk_reasonCode.'\n ARN:'.$xmldata->arn;

		$text_transaction_status_by_notification = $this->language->get('text_transaction_status_by_notification');
		$transactionData = array(
								 'order_id'			=> (int)$order_id,
								 'status'			=> "$state",
								 'state'			=> "$state",
								 'description'		=> "$description",
								 'amount'			=> "$amount",
								 'additional_text'	=> "$text_transaction_status_by_notification"
								 );
		$this->addOrderHistoryStatus($order_id, intval($this->config->get('gpndata_status_chargeback_id')), $description);
		$this->_db->addTransaction($transactionData);
		if(strlen($merchanttransid) == 0 || strlen($transaction_gatetransid) == 0)
			$message = 'Notification data is missing! Fields merchant transaction id or gateway transaction are empty. This needs to be considered as an ERROR! OpenCart Order status has not been updated accordingly!';
		else
			$message = 'Chargeback notification inserted row in transaction log table successfully!';

		$val = null;
		$transactionData = null;

		return $message;
	}

	private function addOrderHistoryStatus($order_id, $status, $description){
		try{
			//if debug mode is enabled, then don't clean anything
			$debug_mode = intval($this->config->get('gpndata_debug_mode'));

			//if transaction was canceled we want to insert void reason entered by user..
			if($status === TransactionStatus::CANCELED){
				$lastOrderTransaction = $this->_db->getLastTransaction($order_id);
				$description = $lastOrderTransaction['additional_text'];
			}

			if($debug_mode == 1){
				$this->model_checkout_order->addOrderHistory($order_id, $status, $description, false);
			}
			else{
				ob_start(); //we need to remove the email warrning error so it does not mess with notification response..
				$this->model_checkout_order->addOrderHistory($order_id, $status, $description, false);
				ob_clean();
			}
		}
		catch (Exception $exception){
			echo $exception;
		}
	}

	public function addOrderHistoryCallback(){
		try{
			if(isset($this->session->data['orderHistoryData']) && $this->request->server['REQUEST_METHOD'] === 'POST'
				&& $this->session->data['token'] == $this->session->data['orderHistoryData']['token']){

				$data = $this->session->data['orderHistoryData'];
				$this->load->model('checkout/order');
				$this->model_checkout_order->addOrderHistory(intval($data['order_id']), $data['status'], $data['description'], false);
				header('Content-Type: application/json');
				echo json_encode(array('order_history_added' => 1));
			}
		}
		catch (Exception $exception){
			echo $exception;
		}
	}
}